#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <pthread.h>
#include <termios.h>
#include <unistd.h>
#include "serial.h"

int verbose = 0;
BoardTypes board_type = BT_powermeter;

FILE *la;

int
getword ()
{
  unsigned int lsb, msb;

  lsb = serial_read ();
  msb = serial_read ();
  return (msb << 8) | lsb;
}

char *
putbits (unsigned char c)
{
  static char buf[9];
  int i;
  for (i=0; i<8; i++)
    {
      buf[i] = c&0x80 ? '1' : '0';
      c <<= 1;
    }
  buf[9] = 0;
  return buf;
}

int
getrecord ()
{
  // Records are:
  // WORD time stamp
  // CHAR data[5] (lsb first)
  // The last record is followed by a time stamp of 0xFFFF

  int timestamp = getword();
  unsigned char data[5];
  int i;

  if (timestamp != 0xffff)
    {
      for (i=0; i<5; i++)
	data[i] = serial_read ();
      fprintf(la, "%5d", timestamp);
      for (i=0; i<5; i++)
	fprintf(la, " %s", putbits(data[i]));
      fprintf(la, "\n");
    }
  return timestamp;
}

static char *
ladata_name ()
{
  static char name[100];
  int i = 0;
  do {
    i ++;
    sprintf(name, "ladata%06d.txt", i);
  } while (access (name, R_OK) == 0);
  printf("%d:", i);
  return name;
}

static void *
console2 (void *x __attribute__((unused)))
{
  int c;
  char *fname;

  serial_set_timeout (-1);
  setbuf (stdout, 0);
  fflush(stdout);
  while (1)
    {
      int count = 0;
      c = serial_read ();
      if (c == 0x80)
	{
	  printf("\033[32m[LA Data:");
	  fname = ladata_name();
	  la = fopen(fname, "w");
	  while (getrecord () != 0xffff)
	    count ++;;
	  fclose (la);
	  printf("%d]\033[0m\r\n", count);
	}
      else
	write (1, &c, 1);
    }
}

static void
do_console ()
{
  struct termios attr, oattr;
  pthread_t tid;
  char c;
  int ctrl_c = 0;

  setbuf (stdout, NULL);

  pthread_create (&tid, 0, console2, 0);

  if (isatty (0))
    {
      tcgetattr(0, &attr);
      tcgetattr(0, &oattr);
      cfmakeraw (&attr);
      tcsetattr(0, TCSANOW, &attr);
    }

  while (read (0, &c, 1) == 1)
    {
      if (c == 3)
	{
	  ctrl_c ++;
	  if (ctrl_c == 2)
	    break;
	}
      else
	ctrl_c = 0;
       serial_write (c);
    }

  pthread_cancel (tid);
  if (isatty (0))
    {
      tcsetattr(0, TCSANOW, &oattr);
    }
}

main()
{
  serial_init ("", 57600);
  serial_change_baud (57600);
  serial_use_cnvss (1);
  serial_change_baud (57600);
  if (! serial_setup_console ())
    {
      fprintf(stderr, "no console support for this board\n");
      exit(1);
    }

  do_console ();
}
